@@ -7,14 +7,15 @@ from importlib.abc import ExecutionLoader
 import importlib.machinery
 import logging
 import os
+from pathlib import Path, PurePath
 import sys
-from tempfile import TemporaryDirectory
 from types import CodeType
-from typing import Any, Dict, List, Optional, Tuple, Union
+from typing import Any, Dict, List, Optional, Set, Tuple, Union
 import opcode
 
-from .common import code_object_replace
-from .module import Module
+from .common import code_object_replace, process_path_specs
+from .common import IncludesList, InternalIncludesList
+from .module import ConstantsModule, Module
 
 
 BUILD_LIST = opcode.opmap["BUILD_LIST"]
@@ -38,17 +39,19 @@ class ModuleFinder:
 
     def __init__(
         self,
-        include_files: Optional[List[Tuple[str, str]]] = None,
+        include_files: Optional[IncludesList] = None,
         excludes: Optional[List[str]] = None,
-        path: Optional[str] = None,
-        replace_paths: Optional[List[str]] = None,
+        path: Optional[List[str]] = None,
+        replace_paths: Optional[List[Tuple[str, str]]] = None,
         zip_include_all_packages: bool = False,
         zip_exclude_packages: Optional[List[str]] = None,
         zip_include_packages: Optional[List[str]] = None,
-        constants_module=None,
-        zip_includes: Optional[List[Tuple[str, str]]] = None,
+        constants_module: Optional[ConstantsModule] = None,
+        zip_includes: Optional[IncludesList] = None,
     ):
-        self.include_files = include_files or []
+        self.include_files: InternalIncludesList = process_path_specs(
+            include_files
+        )
         self.excludes = dict.fromkeys(excludes or [])
         self.optimize_flag = 0
         self.path = path or sys.path
@@ -57,10 +60,12 @@ class ModuleFinder:
         self.zip_exclude_packages = zip_exclude_packages or []
         self.zip_include_packages = zip_include_packages or []
         self.constants_module = constants_module
-        self.zip_includes = zip_includes or []
+        self.zip_includes: InternalIncludesList = process_path_specs(
+            zip_includes
+        )
         self.modules = []
         self.aliases = {}
-        self.exclude_dependent_files = {}
+        self.exclude_dependent_files: Set[Path] = set()
         self._modules: Dict[str, Any] = dict.fromkeys(excludes or [])
         self._builtin_modules = dict.fromkeys(sys.builtin_module_names)
         self._bad_modules = {}
@@ -118,7 +123,7 @@ class ModuleFinder:
             ):
                 module.in_file_system = False
         if module.path is None and path is not None:
-            module.path = path
+            module.path = [Path(p) for p in path]
         if module.file is None and file_name is not None:
             module.file = file_name
         return module
@@ -171,24 +176,19 @@ class ModuleFinder:
         suffixes = importlib.machinery.all_suffixes()
 
         for path in module.path:
-            try:
-                filenames = os.listdir(path)
-            except OSError:
-                continue
-
-            for filename in filenames:
-                fullname = os.path.join(path, filename)
-                if os.path.isdir(fullname):
-                    init_file = os.path.join(fullname, "__init__.py")
-                    if not os.path.exists(init_file):
+            for fullname in path.iterdir():
+                if fullname.is_dir():
+                    init_file = fullname / "__init__.py"
+                    if not init_file.exists():
                         continue
-                    name = filename
+                    name = fullname.name
                 else:
                     # We need to run through these in order to correctly pick
-                    # up PEP 3149 library names (e.g. .cpython-32mu.so).
+                    # up PEP 3149 library names
+                    # (e.g. .cpython-39-x86_64-linux-gnu.so).
                     for suffix in suffixes:
-                        if filename.endswith(suffix):
-                            name = filename[: -len(suffix)]
+                        if fullname.name.endswith(suffix):
+                            name = fullname.name[: -len(suffix)]
 
                             # Skip modules whose names appear to contain '.',
                             # as we may be using the wrong suffix, and even if
@@ -353,7 +353,7 @@ class ModuleFinder:
     def _load_module(
         self,
         name: str,
-        path: Union[str, List[str]],
+        path: Union[str, List[str], List[Path]],
         deferred_imports: DeferredList,
         parent: Optional[Module] = None,
     ) -> Optional[Module]:
@@ -376,6 +376,8 @@ class ModuleFinder:
                 loader = None
         else:
             # Find modules to load
+            if path:
+                path = [str(p) for p in path]
             try:
                 # It's recommended to clear the caches first.
                 importlib.machinery.PathFinder.invalidate_caches()
@@ -507,21 +509,26 @@ class ModuleFinder:
         if code is None:
             code = module.code
         # Prepare the new filename.
-        new_filename = original_filename = os.path.normpath(code.co_filename)
+        original_filename = Path(code.co_filename)
         for search_value, replace_value in self.replace_paths:
             if search_value == "*":
                 if top_level_module.file is None:
                     continue
-                search_value = os.path.dirname(top_level_module.file)
                 if top_level_module.path:
-                    search_value = os.path.dirname(search_value)
-                if search_value:
-                    search_value = search_value + os.path.sep
-            if original_filename.startswith(search_value):
-                new_filename = (
-                    replace_value + original_filename[len(search_value) :]
-                )
+                    search_dir = top_level_module.file.parent.parent
+                else:
+                    search_dir = top_level_module.file.parent
+            else:
+                search_dir = Path(search_value)
+            try:
+                new_filename = original_filename.relative_to(search_dir)
+            except ValueError:
+                pass
+            else:
+                new_filename = replace_value / new_filename
                 break
+        else:
+            new_filename = original_filename
 
         # Run on subordinate code objects from function & class definitions.
         consts = list(code.co_consts)
@@ -532,7 +539,7 @@ class ModuleFinder:
                 )
 
         return code_object_replace(
-            code, co_consts=consts, co_filename=new_filename
+            code, co_consts=consts, co_filename=str(new_filename)
         )
 
     def _run_hook(self, hook: str, module_name: str, *args) -> None:
@@ -627,8 +634,8 @@ class ModuleFinder:
         """
         self.constants_module.values[name] = value
 
-    def ExcludeDependentFiles(self, filename: str) -> None:
-        self.exclude_dependent_files[filename] = None
+    def ExcludeDependentFiles(self, filename: Union[Path, str]) -> None:
+        self.exclude_dependent_files.add(Path(filename))
 
     def ExcludeModule(self, name: str) -> None:
         """
@@ -641,23 +648,27 @@ class ModuleFinder:
             self.excludes[mod] = None
             self._modules[mod] = None
 
-    def IncludeFile(self, path: str, name: Optional[str] = None) -> Module:
+    def IncludeFile(
+        self, path: Union[Path, str], name: Optional[str] = None
+    ) -> Module:
         """Include the named file as a module in the frozen executable."""
         if name is None:
-            name = os.path.splitext(os.path.basename(path))[0]
+            name = Path(path).stem
         deferred_imports: DeferredList = []
-        module = self._load_module(name, path, deferred_imports)
+        module = self._load_module(name, str(path), deferred_imports)
         self._import_deferred_imports(deferred_imports)
         return module
 
     def IncludeFiles(
         self,
-        source_path: str,
-        target_path: str,
+        source_path: Union[Path, str],
+        target_path: Union[Path, str],
         copy_dependent_files: bool = True,
     ) -> None:
         """Include the files in the given directory in the target build."""
-        self.include_files.append((source_path, target_path))
+        self.include_files.extend(
+            process_path_specs([(source_path, target_path)])
+        )
         if not copy_dependent_files:
             self.ExcludeDependentFiles(source_path)
 
@@ -710,8 +721,12 @@ class ModuleFinder:
             self.optimize_flag = optimize_flag
         return previous
 
-    def ZipIncludeFiles(self, source_path, target_path):
-        """
-        Include files or all of the files in a directory to the zip file.
-        """
-        self.zip_includes.append((source_path, target_path))
+    def ZipIncludeFiles(
+        self,
+        source_path: Union[str, Path],
+        target_path: Optional[Union[str, Path, PurePath]] = None,
+    ) -> None:
+        """Include files or all of the files in a directory to the zip file."""
+        self.zip_includes.extend(
+            process_path_specs([(source_path, target_path)])
+        )
